@@ -18,6 +18,7 @@ use ChordPro::Paths;
 use ChordPro::Utils qw( expand_tilde demarkup min is_corefont );
 use ChordPro::Output::Common qw( fmt_subst prep_outlines );
 use File::LoadLines qw(loadlines);
+use Ref::Util qw( is_hashref );
 
 # For regression testing, run perl with PERL_HASH_SEED set to zero.
 # This eliminates the arbitrary order of font definitions and triggers
@@ -413,7 +414,7 @@ sub add_object {
     my $w = $o->width  * $scale_x;
     my $h = $o->height * $scale_y;
 
-    warn( sprintf("add_object x=%.1f y=%.1f w=%.1f h=%.1f scale=%.1f,%.1f) %s\n",
+    warn( sprintf("add_object x=%.1f y=%.1f w=%.1f h=%.1f scale=%.1f,%.1f %s\n",
 		  $x, $y, $w, $h, $scale_x, $scale_y, $ha,
 		 ) ) if $config->{debug}->{images};
 
@@ -736,7 +737,7 @@ sub init_fonts {
 	my @fam = split( /\s*,\s*/, $ff );
 	foreach my $s ( keys( %{ $ps->{fontconfig}->{$ff} } ) ) {
 	    my $v = $ps->{fontconfig}->{$ff}->{$s};
-	    if ( UNIVERSAL::isa( $v, 'HASH' ) ) {
+	    if ( is_hashref($v) ) {
 		my $file = delete( $v->{file} );
 		$fc->register_font( $file, $fam[0], $s, $v );
 	    }
@@ -785,7 +786,8 @@ sub init_pangofont {
 	$font->{_ff} = $ff;
 	$font->{fd}->set_shaping( $font->{fd}->get_shaping || $font->{shaping}//0);
 	$font->{size} = $font->{fd}->get_size if $font->{fd}->get_size;
-    };
+	1;
+    } or return;
     $font->{fd};
 }
 
