@@ -21,8 +21,8 @@ sub get( $class, $reset = 0 ) {
 }
 
 use Cwd qw(realpath);
+use File::Spec::Functions qw( catfile catdir splitpath catpath file_name_is_absolute );
 use File::HomeDir;
-use ChordPro::Files;
 
 field $home      :reader;	# dir
 field $configdir :reader;	# dir
@@ -40,7 +40,7 @@ BUILD {
     my $app = "ChordPro";
     my $app_lc = lc($app);
 
-    $pathsep = is_msw ? ';' : ':';
+    $pathsep = $self->is_msw ? ';' : ':';
 
     $home     = realpath( $ENV{HOME} = File::HomeDir->my_home );
 
@@ -69,27 +69,27 @@ BUILD {
     if ( defined( $ENV{XDG_CONFIG_HOME} ) && $ENV{XDG_CONFIG_HOME} ne "" ) {
 	push( @try,
 	      # See https://specifications.freedesktop.org/basedir-spec/basedir-spec-latest.html
-	      # fn_catdir( $ENV{XDG_CONFIG_HOME}, ".config", $app_lc ),
-	      # fn_catdir( $ENV{XDG_CONFIG_HOME}, ".config" ),
-	      # fn_catdir( $ENV{XDG_CONFIG_HOME}, ".$app_lc" ) );
-	      fn_catdir( $ENV{XDG_CONFIG_HOME}, "$app_lc" ) );
+	      # catdir( $ENV{XDG_CONFIG_HOME}, ".config", $app_lc ),
+	      # catdir( $ENV{XDG_CONFIG_HOME}, ".config" ),
+	      # catdir( $ENV{XDG_CONFIG_HOME}, ".$app_lc" ) );
+	      catdir( $ENV{XDG_CONFIG_HOME}, "$app_lc" ) );
     }
     else {
 	push( @try,
-	      fn_catdir( $home, ".config", $app_lc ),
-	      fn_catdir( $home, ".$app_lc" ),
+	      catdir( $home, ".config", $app_lc ),
+	      catdir( $home, ".$app_lc" ),
 	      File::HomeDir->my_dist_config($app) );
     }
 
     for ( @try ) {
-	next unless $_ && fs_test( d => $_);
+	next unless $_ && -d $_;
 	my $path = $self->normalize($_);
 	warn("Paths: configdir try $_ => $path\n") if $self->debug > 1;
-	next unless $path && fs_test( d => $path);
+	next unless $path && -d $path;
 	$configdir = $path;
-	for ( $self->normalize( fn_catfile( $path, "$app_lc.prp" ) ),
-	      $self->normalize( fn_catfile( $path, "$app_lc.json" ) ) ) {
-	    next unless $_ && fs_test( f => $_ );
+	for ( $self->normalize( catfile( $path, "$app_lc.prp" ) ),
+	      $self->normalize( catfile( $path, "$app_lc.json" ) ) ) {
+	    next unless $_ && -f $_;
 	    $configs->{userconfig} = $_;
 	    last;
 	}
@@ -99,7 +99,7 @@ BUILD {
 
     for ( $self->normalize(".$app_lc.json"),
 	  $self->normalize("$app_lc.json") ) {
-	    next unless $_ && fs_test( f => $_ );
+	    next unless $_ && -f $_;
 	$configs->{config} = $_;
 	last;
     }
@@ -117,7 +117,7 @@ BUILD {
     $self->setup_resdirs;
 
     # Check for packaged image.
-    for ( qw( OCI Docker AppImage PPL ) ) {
+    for ( qw( Docker AppImage PPL ) ) {
 	next unless exists $ENV{uc($_)."_PACKAGED"}
 	  && $ENV{uc($_)."_PACKAGED"};
 	$packager = $_;
@@ -140,7 +140,7 @@ method setup_resdirs {
 	next unless $_;
 	my $path = $self->normalize($_);
 	warn("Paths: resdirs try $_ => $path\n") if $self->debug > 1;
-	next unless $path && fs_test( d => $path );
+	next unless $path && -d $path;
 	push( @$resdirs, $path );
     }
 
@@ -161,16 +161,20 @@ method debug {
     $ENV{CHORDPRO_DEBUG_PATHS} || $::config->{debug}->{paths} || 0;
 }
 
+method is_msw {
+    $^O =~ /mswin/i;
+}
+
 # Is absolute.
 
 method is_absolute ( $p ) {
-    fn_is_absolute( $p );
+    File::Spec->file_name_is_absolute( $p );
 }
 
 # Is bare (no volume/dir).
 
 method is_here ( $p ) {
-    my ( $v, $d, $f ) = fn_splitpath($p);
+    my ( $v, $d, $f ) = splitpath($p);
     $v eq '' && $d eq '';
 }
 
@@ -197,20 +201,16 @@ method path ( $p = undef ) {
 	local $ENV{PATH} = $p;
 	my @p = File::Spec->path();
 	# On MSWindows, '.' is always prepended.
-	shift(@p) if is_msw;
+	shift(@p) if $self->is_msw;
 	return @p;
     }
     return File::Spec->path();
 }
 
-# Prepend/append dirs to path.
+# Prepend dirs in front of path.
 
 method pathprepend( @d ) {
-    $ENV{PATH} = $self->pathcombine( @d, $ENV{PATH} );
-}
-
-method pathappend( @d ) {
-    $ENV{PATH} = $self->pathcombine( $ENV{PATH}, @d );
+    $ENV{PATH} = join( $pathsep, @d, $ENV{PATH} );
 }
 
 method pathcombine( @d ) {
@@ -222,28 +222,15 @@ method pathcombine( @d ) {
 method findexe ( $p, %opts ) {
     my $try = $p;
     my $found;
-    if ( is_msw ) {
+    if ( $self->is_msw ) {
 	$try .= ".exe";
     }
-
-    if ( fn_is_absolute($p)
-	 && ChordPro::Files::fs_test( fx => $p ) ) {
-	warn("Paths: findexe $p => ", $self->display($p), "\n")
-	  if $self->debug;
-	return $p;
-    }
-
     for ( $self->path ) {
-	my $e = fn_catfile( $_, $try );
-	$found = realpath($e), last if fs_test( fx => $e );
-    }
-    if ( $self->debug ) {
-	warn("Paths: findexe $p => ", $self->display($found), "\n");
-    }
-    elsif ( !$found && !$opts{silent} ) {
-	warn("Could not find $p in ",
-	     join( " ", map { qq{"$_"} } $self->path ), "\n");
+	my $e = catfile( $_, $try );
+	$found = realpath($e), last if -f -x $e;
     }
+    warn("Paths: findexe $p => ", $self->display($found), "\n")
+      if $self->debug;
     return $found;
 }
 
@@ -253,7 +240,7 @@ method findcfg ( $p ) {
     my $found;
     my @p;
     if ( $p =~ /\.\w+$/ ) {
-	$found = realpath($p) if fs_test( fs => $p );
+	$found = realpath($p) if -f -s $p;
 	@p = ( $p );
     }
     else {
@@ -263,8 +250,8 @@ method findcfg ( $p ) {
     unless ( $found ) {
 	for ( @$resdirs ) {
 	    for my $cfg ( @p ) {
-		my $f = fn_catfile( $_, "config", $cfg );
-		$found = realpath($f), last if fs_test( fs => $f );
+		my $f = catfile( $_, "config", $cfg );
+		$found = realpath($f), last if -f -s $f;
 	    }
 	}
     }
@@ -278,16 +265,16 @@ method findcfg ( $p ) {
 method findres ( $p, %opts ) {
     my $try = $p;
     my $found;
-    if ( fn_is_absolute($p) ) {
+    if ( file_name_is_absolute($p) ) {
 	$found = realpath($p);
     }
     else {
 	if ( defined $opts{class} ) {
-	    $try = fn_catfile( $opts{class}, $try );
+	    $try = catfile( $opts{class}, $try );
 	}
 	for ( @$resdirs ) {
-	    my $f = fn_catfile( $_, $try );
-	    $found = realpath($f), last if fs_test( fs => $f );
+	    my $f = catfile( $_, $try );
+	    $found = realpath($f), last if -f -s $f;
 	}
     }
     warn("Paths: findres", $opts{class} ? " [$opts{class}]" : "",
@@ -302,11 +289,11 @@ method findresdirs ( $p, %opts ) {
     my $try = $p;
     my @found;
     if ( defined $opts{class} ) {
-	$p = fn_catdir( $opts{class}, $p );
+	$p = catdir( $opts{class}, $p );
     }
     for ( @$resdirs ) {
-	my $d = fn_catdir( $_, $p );
-	push( @found, realpath($d) ) if fs_test( d => $d );
+	my $d = catdir( $_, $p );
+	push( @found, realpath($d) ) if -d $d;
     }
     if ( $self->debug ) {
 	my $i = 0;
@@ -324,10 +311,10 @@ method findresdirs ( $p, %opts ) {
 
 method sibling ( $orig, %opts ) {
     # Split.
-    my ( $v, $d, $f ) = fn_splitpath($orig);
+    my ( $v, $d, $f ) = splitpath($orig);
     my $res;
     if ( $opts{name} ) {
-	$res = fn_catpath( $v, $d, $opts{name} );
+	$res = catpath( $v, $d, $opts{name} );
     }
     else {
 	# Get base and extension.
@@ -339,7 +326,7 @@ method sibling ( $orig, %opts ) {
 	$f = $b;
 	$f .= $e if defined $e;
 	# Join with path.
-	$res = fn_catpath( $v, $d, $f );
+	$res = catpath( $v, $d, $f );
     }
     warn("Paths: sibling $orig => ", $self->display($res), "\n")
       if $self->debug;
@@ -351,7 +338,7 @@ method sibling ( $orig, %opts ) {
 method siblingres ( $orig, $name, %opts ) {
     return unless defined $orig;
     my $try = $self->sibling( $orig, name => $name );
-    my $found = ( $try && fs_test( s => $try ) )
+    my $found = ( $try && -s $try )
       ? $try
       : $self->findres( $name, class => $opts{class} );
     return $found;
