@@ -13,11 +13,13 @@ use warnings;
 use ChordPro;
 use ChordPro::Config;
 use ChordPro::Song;
+use ChordPro::Utils qw(progress);
 
 use Carp;
 use List::Util qw(any);
 use File::LoadLines;
 use Storable qw(dclone);
+use Ref::Util qw(is_arrayref is_plain_hashref);
 
 sub new {
     my ($pkg) = @_;
@@ -36,7 +38,9 @@ sub parse_file {
     }
 
     # Loadlines sets $opts->{_filesource}.
-    my $lines = loadlines( $filename, $opts );
+    $opts->{fail} = "soft";
+    my $lines = is_arrayref($filename) ? $filename : loadlines( $filename, $opts );
+    die( $filename, ": ", $opts->{error}, "\n" ) if $opts->{error};
     # Sense crd input and convert if necessary.
     if ( !(defined($options->{a2crd}) && !$options->{a2crd}) and
 	 !$options->{fragment}
@@ -59,7 +63,7 @@ sub parse_file {
 	next unless exists $opts->{$_};
 	$options->{$_} = $opts->{$_};
     }
-    bless $config => ChordPro::Config:: if ref $config eq 'HASH';
+    bless $config => ChordPro::Config:: if is_plain_hashref($config);
 
     my $linecnt = 0;
     my $songs = 0;
@@ -96,6 +100,12 @@ sub parse_file {
     return 1;
 }
 
+sub add {
+    my ( $self, $song ) = @_;
+    push( @{$self->{songs}}, $song );
+    $self;
+}
+
 sub embed_file {
     my ( $self, $filename, $meta, $defs ) = @_;
 
@@ -105,7 +115,7 @@ sub embed_file {
     }
     my $type = "pdf";
 
-    my $song = ChordPro::Song->new( $filename );
+    my $song = ChordPro::Song->new( { filesource => $filename } );
     $song->{meta}->{songindex} = 1 + @{ $self->{songs} };
     $song->{source} =
       { file      => $filename,
