@@ -12,12 +12,14 @@ class ChordPro::Wx::EditorPanel
 use Wx qw[:everything];
 use Wx::Locale gettext => '_T';
 
-use ChordPro::Files;
+use ChordPro::Utils qw( is_macos );
 use ChordPro::Wx::Config;
 use ChordPro::Wx::Utils;
-use ChordPro::Utils qw( max demarkup plural );
+use ChordPro::Utils qw( max demarkup is_macos is_msw plural );
 use ChordPro::Paths;
 
+use File::Basename;
+
 # WhoamI
 field $panel :accessor = "editor";
 
@@ -55,11 +57,6 @@ method refresh() {
 
     $self->update_menubar( M_EDITOR );
 
-    # Flush pending messages.
-    if ( $state{msgs} ) {
-	$self->log( 'I', $_ ) for @{$state{msgs}};
-	$state{msgs} = [];
-    }
     $self->log( 'I', "Using " .
 		( $state{have_stc}
 		  ? "styled" : "basic") . " text editor" );
@@ -84,6 +81,7 @@ method refresh() {
     $self->previewtooltip;
     $self->messagestooltip;
     $self->{t_editor}->SetModified($mod);
+    $self->{bmb_preview}->SetFocus;
 
     $self->refresh_messages;
 
@@ -93,14 +91,14 @@ method refresh() {
 	Wx::Event::EVT_STC_CLIPBOARD_PASTE( $self, $self->{t_editor}->GetId,
 					    $self->can("OnClipBoardPaste") );
     }
-    $self->set_focus;
+
 }
 
 method openfile( $file, $checked=0, $actual=undef ) {
     $actual //= $file;
 
-    # Bypass test when already checked. TODO?
-    unless ( $checked || fs_test( 'fr', $file ) ) {
+    # File tests fail on Windows, so bypass when already checked.
+    unless ( $checked || -f -r $file ) {
 	$self->log( 'W',  "Error opening $file: $!",);
 	my $md = Wx::MessageDialog->new
 	  ( $self,
@@ -111,20 +109,7 @@ method openfile( $file, $checked=0, $actual=undef ) {
 	$md->Destroy;
 	return;
     }
-    if ( my $f = fs_load($file) ) {
-
-	# Check for filelist.
-	if ( $f->[0] =~ m;^//\s*chordpro\s+songbook;i ) {
-	    $f->[0] = $file;
-	    return $self->GetParent->select_mode("sbexport")->load_filelist($f)
-	}
-
-	# This has the (desired) sideeffect that all newlines
-	# are now \n .
-	$self->{t_editor}->SetText(join("\n",@$f)."\n");
-	$self->{t_editor}->DiscardEdits;
-    }
-    else {
+    unless ( $self->{t_editor}->LoadFile($file) ) {
 	$self->log( 'W',  "Error opening $file: $!",);
 	my $md = Wx::MessageDialog->new
 	  ( $self,
@@ -170,7 +155,7 @@ method openfile( $file, $checked=0, $actual=undef ) {
     }
     else {
 	my $n = $self->{t_editor}->GetLineCount;
-	$self->{l_status}->SetLabel(fn_basename($file));
+	$self->{l_status}->SetLabel(basename($file));
 	$self->{l_status}->SetToolTip($file);
 	$self->log( 'S', "Loaded: $file (" . plural($n, " line") . ")");
     }
@@ -220,16 +205,32 @@ method newfile( $file = undef ) {
     my $file = $preferences{tmplfile};
     if ( $file && $preferences{enable_tmplfile} ) {
 	$self->log( 'I', "Loading template $file" );
-	if ( fs_test( fr => $file ) && $self->{t_editor}->LoadFile($file) ) {
+	if ( -f -r $file && $self->{t_editor}->LoadFile($file) ) {
 	    $content = "";
 	}
 	else {
 	    $self->log( 'E', "Cannot open template $file: $!" );
 	}
     }
-    else {
+    elsif ( 1 ) {
 	$content = "{title: $title}";
     }
+    else {
+	require ChordPro::Wx::NewSongDialog;
+	unless ( $self->{d_newfile} ) {
+	    $self->{d_newfile} = ChordPro::Wx::NewSongDialog->new
+	      ( $self, wxID_ANY, $title );
+	    restorewinpos( $self->{d_newfile}, "newfile" );
+	    $self->{d_newfile}->set_title($title);
+	}
+	$self->{d_newfile}->refresh;
+	my $ret = $self->{d_newfile}->ShowModal;
+	savewinpos( $self->{d_newfile}, "newfile" );
+	if ( $ret == wxID_OK ) {
+	    $title = $self->{d_newfile}->get_title;
+	    $content = $self->{d_newfile}->get_meta;
+	}
+    }
 
     for ( $self->{t_editor} ) {
 	$content =~ s/[\n\r]*\Z//;
@@ -271,7 +272,7 @@ method check_source_saved() {
     if ( $state{currentfile} ) {
 	my $md = Wx::MessageDialog->new
 	  ( $self,
-	    "File " . fn_basename($state{currentfile}) . " has been changed.\n".
+	    "File " . basename($state{currentfile}) . " has been changed.\n".
 	    "Do you want to save your changes?",
 	    "File has changed",
 	    0 | wxCANCEL | wxYES_NO | wxYES_DEFAULT | wxICON_QUESTION );
@@ -307,10 +308,9 @@ method check_source_saved() {
 method save_file( $file = undef ) {
     while ( 1 ) {
 	unless ( defined $file && $file ne "" ) {
-	    my $cf = $state{currentfile} // "Untitled";
 	    my $fd = Wx::FileDialog->new
 	      ($self, _T("Choose output file"),
-	       fn_dirname($cf), fn_basename($cf),
+	       "", $state{currentfile}//"",
 	       "*".$preferences{chordproext},
 	       0|wxFD_SAVE|wxFD_OVERWRITE_PROMPT,
 	       wxDefaultPosition);
@@ -321,28 +321,17 @@ method save_file( $file = undef ) {
 	    $fd->Destroy;
 	}
 	return unless defined $file;
-
-	# On macOS Catalina DMG STC seems to have problems saving.
-	my $fd = fs_open( $file, '>:utf8' );
-	$self->{t_editor}->ConvertEOLs(wxSTC_EOL_LF);
-	my $t = $self->{t_editor}->GetText;
-	# $self->log( 'I', ChordPro::Utils::as($t));
-	$t .= "\n" unless $t =~ /\n$/;
-	if ( $fd
-	     and print $fd $t
-	     and $fd->close ) {
+	if ( $self->{t_editor}->SaveFile($file) ) {
 	    $self->{t_editor}->SetModified(0);
 	    $state{currentfile} = $file;
 	    $state{windowtitle} = $file;
 	    $self->log( 'S',  "Saved: $file" );
-	    use List::Util qw(uniq);
-	    @{$state{recents}} = uniq( $file, @{$state{recents}} );
 	    return;
 	}
 
 	my $md = Wx::MessageDialog->new
 	  ( $self,
-	    "Cannot save to $file\n$!",
+	    "Cannot save to $file",
 	    "Error saving file",
 	    0 | wxOK | wxICON_ERROR);
 	$md->ShowModal;
@@ -362,9 +351,9 @@ method preview( $args, %opts ) {
     # The text that we get from the editor can have CRLF line endings,
     # that on Windows will result in double line ends. Write with
     # 'raw' layer.
-    require Encode;
-    if ( $fd = fs_open( $preview_cho, '>:raw' )
-	 and print $fd ( Encode::encode_utf8($self->{t_editor}->GetText) )
+    use Encode 'encode_utf8';
+    if ( open( $fd, '>:raw', $preview_cho )
+	 and print $fd ( encode_utf8($self->{t_editor}->GetText) )
 	 and close($fd) ) {
 	$self->prv->preview( $args, %opts );
 	$self->previewtooltip;
@@ -375,9 +364,6 @@ method preview( $args, %opts ) {
 }
 
 method check_preview_saved() {
-    # Do not ask for preview save. It's regenerated easily.
-    return 1;
-
     return 1 unless $self->prv && $self->prv->unsaved_preview;
 
     my $md = Wx::MessageDialog->new
@@ -476,48 +462,19 @@ method update_preferences() {
     $self->refresh;
 }
 
-method set_focus {
-    $self->{t_editor}->SetFocus;
-}
-
 ################ Event Handlers (alphabetic order) ################
 
-method OnInsertSymbol($event) {
-    unless ( $preferences{enable_insert_symbols} ) {
-	my $md = Wx::MessageDialog->new
-	  ( undef,
-	    "Note: some symbols might not display properly in PDF".
-	    " output, even if visible in the Editor. Make sure that".
-	    " the symbols you use are supported by your output fonts.\n".
-	    "\n".
-	    "Continue and suppress future warnings?",
-	    "Advanced operation warning",
-	    wxYES_NO|wxNO_DEFAULT|wxICON_WARNING|wxDIALOG_NO_PARENT );
-	return unless $md->ShowModal == wxID_YES;
-	$preferences{enable_insert_symbols} = 1;
-
-    }
-    my $ctrl = $self->{t_editor};
-    state $sym = "\x{2665}";
-    my $d = Wx::SymbolPickerDialog->new( $sym, "", "", $self );
-    if ( $d->ShowModal == wxID_OK ) {
-	$ctrl->AddText( $sym = $d->GetSymbol );
-    }
-}
-
 method OnA2Crd($event) {
 
     my $ctrl = $self->{t_editor};
     my ( $from, $to ) = $ctrl->GetSelection;
     my $have_selection = $from != $to;
-    $ctrl->ConvertEOLs(wxSTC_EOL_LF);
     my $text = $have_selection ? $ctrl->GetSelectedText : $ctrl->GetText;
 
     require ChordPro::A2Crd;
     $::options->{nosysconfig} = 1;
     $::options->{nouserconfig} = 1;
     $::options->{noconfig} = 1;
-    $::options->{fragment} = $have_selection;
 
     # Often text that is pasted from web has additional newlines.
     $text =~ s/^\n+//;
@@ -574,9 +531,6 @@ method OnCharAdded( $event ) {
 	    $stc->CharRightExtend if length($nl) == 2;
 	    $stc->ReplaceSelection("}" . $nl);
 	}
-	elsif ( $ln > 1 && $stc->GetLine($ln-1) =~ /^\{\s*start_of_(\w+).*\}$/ ) {
-	    $stc->InsertText( -1, "\n{end_of_$1}" );
-	}
     }
 
     elsif ( $key eq ord(" ") || $key eq ord(":") || $key eq ord("}") ) {
@@ -593,7 +547,7 @@ method OnCharAdded( $event ) {
     }
 }
 
-method OnClearDiagnosticFlags($event) {
+method OnClearAnnotations($event) {
     return unless $state{have_stc};
     $self->{t_editor}->AnnotationClearAll;
 }
@@ -620,8 +574,6 @@ method OnCloseSection($event) {
 		$closed = "";
 		next;
 	    }
-	    $stc->AddText( $self->nl )
-	      if $stc->GetColumn($stc->GetCurrentPos);
 	    $stc->AddText( "{$1end_of_$2}" . $self->nl );
 	    $did++;
 	    last;
@@ -631,8 +583,6 @@ method OnCloseSection($event) {
 		$closed = "";
 		next;
 	    }
-	    $stc->AddText( $self->nl )
-	      if $stc->GetColumn($stc->GetCurrentPos);
 	    $stc->AddText( "{$1eo$2}" . $self->nl );
 	    $did++;
 	    last;
@@ -760,10 +710,6 @@ method OnInsertGrid($event) {
     $self->embrace_section("grid");
 }
 
-method OnInsertTab($event) {
-    $self->embrace_section("tab");
-}
-
 method OnInsertSection($event) {
     $self->embrace_section(undef);
 }
