@@ -9,11 +9,12 @@ role ChordPro::Wx::PanelRole;
 use Wx qw[:everything];
 use Wx::Locale gettext => '_T';
 
-use ChordPro::Files;
-use ChordPro::Paths;
-use ChordPro::Utils qw( demarkup plural );
 use ChordPro::Wx::Config;
 use ChordPro::Wx::Utils;
+use ChordPro::Utils qw( demarkup is_macos is_msw plural );
+use ChordPro::Paths;
+
+use File::Basename;
 
 # Either Wx::WebView or Wx::StaticText.
 field $wv			:accessor;
@@ -206,10 +207,9 @@ method OnPreviewMore($event) {
     push( @args, "--transpose=$n" );
 
     my $i = 0;
-    while ( exists $d->{"cb_customtask_$i"} ) {
+    for ( @{$state{tasks}} ) {
 	if ( $d->{"cb_customtask_$i"}->IsChecked ) {
-	    push( @args, "--config",
-		  $state{presets}{tasks}{lc $d->{"cb_customtask_$i"}->GetLabel}->{file} );
+	    push( @args, "--config", $_->[1] );
 	}
 	$i++;
     }
@@ -284,7 +284,8 @@ method OnMessagesSave($event) {
     my $fd = Wx::FileDialog->new
       ( $self,
 	_T("Choose file to save in"),
-	fn_dirname($file), fn_basename($file),
+	"",
+	$file,
 	"*",
 	wxFD_SAVE|wxFD_OVERWRITE_PROMPT );
 
@@ -328,7 +329,6 @@ method previewtooltip() {
 	$self->{bmb_preview}->SetToolTip(_T("Generate and show a new preview"));
 	$mi->Check(0);
     }
-    $self->panel_focus;
 }
 
 method messagestooltip() {
@@ -343,18 +343,6 @@ method messagestooltip() {
 	$mi->Check(0);
     }
     $self->{w_infobar}->Dismiss if $self->{w_infobar}->IsShown;
-    $self->panel_focus;
-    $self->panel_linenums( $mi->IsChecked );
-}
-
-method panel_focus() {
-    return unless UNIVERSAL::can( $state{panel}, "set_focus" );
-    $state{panel}->set_focus;
-}
-
-method panel_linenums( $b ) {
-    return unless UNIVERSAL::can( $state{panel}, "showlinenumbers" );
-    $state{panel}->showlinenumbers($b);
 }
 
 1;
