@@ -8,30 +8,29 @@ use Carp;
 class ChordPro::Wx::Preview;
 
 use ChordPro;
-use ChordPro::Files;
 use ChordPro::Paths;
 use ChordPro::Wx::Config;
 use ChordPro::Utils qw( demarkup );
 
 use Wx ':everything';
 use Wx::Locale gettext => '_T';
-use File::Temp qw( tempfile tempdir );
+
+use File::Temp qw( tempfile );
 use File::Basename qw(basename);
-use Ref::Util qw( is_hashref );
 
 field $panel			:param;
 field $msgs;
 field $fatal;
 field $died;
-field $tmpdir;
 field $preview_cho		:accessor;
-field $preview_file;
+field $preview_pdf;
 field $preview_tmp;
 field $unsaved_preview		:mutator;
 
 BUILD {
-    $tmpdir = tempdir( CLEANUP => 1 );
-    $preview_cho = fn_catfile( $tmpdir, "preview.cho" );
+    ( undef, $preview_cho ) = tempfile( OPEN => 0 );
+    $preview_pdf = $preview_cho . ".pdf";
+    $preview_cho .= ".cho";
 }
 
 method _warn( @m ) {
@@ -56,16 +55,13 @@ method log( $level, $msg ) {
 
 method preview( $args, %opts ) {
 
-    unlink($preview_file) if $preview_file;
+    unlink($preview_pdf);
 
     my $annotate = eval { $panel->prepare_annotations };
 
     #### ChordPro
 
     @ARGV = ();			# just to make sure
-    push( @ARGV, "--debug" ) if $state{debug};
-    push( @ARGV, "--trace" ) if $state{trace};
-    push( @ARGV, "--verbose" ) for 1..($state{verbose}//0);
 
     $msgs = $fatal = $died = 0;
     local $SIG{__WARN__} = sub {
@@ -85,54 +81,39 @@ method preview( $args, %opts ) {
     if ( $preferences{skipstdcfg} ) {
 	push( @ARGV, '--nodefaultconfigs' );
     }
-
-    for my $preset ( qw( instruments styles stylemods ) ) {
-	for my $p ( @{$preferences{"preset_$preset"}} ) {
-	    next if $p->{default};
-	    next unless defined $p->{file};
-	    push( @ARGV, "--config", $p->{file} );
+    if ( $preferences{enable_presets} && $preferences{cfgpreset} ) {
+	# Only include the ones we have.
+	my %s = ( map { $_ => 1 } @{$state{styles}}, @{$state{userstyles}} );
+	foreach ( @{ $preferences{cfgpreset} } ) {
+	    next unless exists($s{$_});
+	    push( @ARGV, '--config', $_ );
 	    $haveconfig++;
 	}
     }
-
     if ( $preferences{enable_configfile} ) {
 	$haveconfig++;
 	push( @ARGV, '--config', $preferences{configfile} );
 
     }
-    local $ENV{CHORDPRO_LIB};
+    delete $ENV{CHORDPRO_LIB};
     if ( $preferences{enable_customlib} ) {
 	$ENV{CHORDPRO_LIB} = $preferences{customlib};
     }
     CP->setup_resdirs;
-
-    if ( $preferences{enable_xcode} ) {
-	my $c = $preferences{preset_xcodes}[0];
-	unless ( $c->{default} ) {
-	    $haveconfig++;
-	    push( @ARGV, '--transcode', $c->{system} );
-	}
+    if ( $preferences{enable_xcode} && $preferences{xcode} ) {
+	$haveconfig++;
+	push( @ARGV, '--transcode', $preferences{xcode} );
     }
 
-    if ( $preferences{preset_notations} ) {
-	my $c = $preferences{preset_notations}[0];
-	unless ( $c->{default} ) {
-	    $haveconfig++;
-	    push( @ARGV, '--config', $c->{file} );
-	}
+    if ( $preferences{notation} ) {
+	$haveconfig++;
+	push( @ARGV, '--config', 'notes:' . $preferences{notation} );
     }
 
     push( @ARGV, '--noconfig' ) unless $haveconfig;
 
-    if ( $preferences{enable_htmlviewer} ) {
-	$preview_file = fn_catfile( $tmpdir, "preview.html" );
-	push( @ARGV, '--generate', "HTML" );
-    }
-    else {
-	$preview_file = fn_catfile( $tmpdir, "preview.pdf" );
-	push( @ARGV, '--generate', "PDF" );
-    }
-    push( @ARGV, '--output', $preview_file );
+    push( @ARGV, '--output', $preview_pdf );
+    push( @ARGV, '--generate', "PDF" );
 
     push( @ARGV, '--transpose', $state{xpose} )
 	 if $preferences{enable_xpose} && $state{xpose};
@@ -182,12 +163,6 @@ method preview( $args, %opts ) {
 	      return 1;
 	  } );
 
-    require Encode;
-    for ( @ARGV ) {
-	next if ref ne "";
-#	$_ = Encode::encode_utf8($_);
-    }
-
     eval {
 	$options = ChordPro::app_setup( "ChordPro", $ChordPro::VERSION );
     };
@@ -207,15 +182,9 @@ method preview( $args, %opts ) {
     };
     $dialog->Destroy if $dialog;
     $self->_die($@), goto ERROR if $@ && !$died;
-    goto ERROR unless fs_test( e => $preview_file );
+    goto ERROR unless -e $preview_pdf;
 
     $unsaved_preview = 1;
-    if ( $preferences{enable_htmlviewer} ) {
-	fs_copy( CP->findres( $_, class => "styles" ),
-		 fn_catfile( $tmpdir, $_ ) )
-	  for qw( chordpro.css chordpro_print.css );
-    }
-
     if ( !$preferences{enable_pdfviewer}
 	 && $panel->{webview}->isa('Wx::WebView') ) {
 
@@ -227,7 +196,7 @@ method preview( $args, %opts ) {
 	    unless ( $_->{sw_lr}->IsSplit ) {
 		$_->{sw_lr}->SplitVertically ( $_->{p_left},
 					       $_->{p_right},
-					       $state{sash}{$_->panel."_lr"} // 0.5 );
+					       $_->{$panel->panel."_lr"} // 0.5 );
 	    }
 	}
 
@@ -237,7 +206,7 @@ method preview( $args, %opts ) {
 	    ( $panel, $panel->{webview}, $self->can("OnWebViewError") );
 
 	use URI::file;
-	my $wf = URI::file->new($preview_file);
+	my $wf = URI::file->new($preview_pdf);
 	$wf =~ s;///([A-Z]):/;///$1|/;;
 	$self->log( 'I', "Preview " . substr($wf,0,128) );
 	$panel->{webview}->LoadURL($wf);
@@ -246,26 +215,25 @@ method preview( $args, %opts ) {
 	$self->log( 'S', "Output generated, starting previewer");
 
 	if ( my $cmd = $preferences{pdfviewer} ) {
-	    if ( $cmd =~ s/\%f/$preview_file/g ) {
+	    if ( $cmd =~ s/\%f/$preview_pdf/g ) {
 	    }
 	    elsif ( $cmd =~ /\%u/ ) {
-		my $u = _makeurl($preview_file);
+		my $u = _makeurl($preview_pdf);
 		$cmd =~ s/\%u/$u/g;
 	    }
 	    else {
-		$cmd .= " \"$preview_file\"";
+		$cmd .= " \"$preview_pdf\"";
 	    }
 	    Wx::ExecuteCommand($cmd);
 	}
 	else {
 	    my $wxTheMimeTypesManager = Wx::MimeTypesManager->new;
-	    my $ft = $wxTheMimeTypesManager->GetFileTypeFromExtension
-	      ( $preferences{enable_htmlviewer} ? "html" : "pdf");
-	    if ( $ft && ( my $cmd = $ft->GetOpenCommand($preview_file) ) ) {
+	    my $ft = $wxTheMimeTypesManager->GetFileTypeFromExtension("pdf");
+	    if ( $ft && ( my $cmd = $ft->GetOpenCommand($preview_pdf) ) ) {
 		Wx::ExecuteCommand($cmd);
 	    }
 	    else {
-		Wx::LaunchDefaultBrowser($preview_file);
+		Wx::LaunchDefaultBrowser($preview_pdf);
 	    }
 	}
     }
@@ -286,7 +254,7 @@ method preview( $args, %opts ) {
 	    $self->log( 'W',  "Problems found." );
 	}
     }
-    elsif ( !fs_test( s => $preview_file ) ) {
+    elsif ( ! -s $preview_pdf ) {
 	$panel->alert(1);
 	$self->log( 'W',  "Nothing to view. Empty song?" );
     }
@@ -329,54 +297,31 @@ sub _makeurl {
 }
 
 method save {
-    return unless fs_test( s => $preview_file );
-
-    if ( $preferences{enable_htmlviewer} ) {
-	return $panel->{webview}->Print;
-    }
-
-    my $savefile = "preview";
-    if ( $state{mode} eq "editor" && $state{currentfile} ) {
-	$savefile = $state{currentfile} =~ s/\.\w+$//r;
-    }
-    if ( $state{mode} eq "sbexport" && $state{sbe_folder} ) {
-	$savefile = $state{sbe_folder} . ".pdf";
-    }
-
+    return unless -s $preview_pdf;
     my $fd = Wx::FileDialog->new
       ( $panel,
 	_T("Choose output file"),
-	fn_dirname($savefile), fn_basename($savefile),
+	"", "preview",
 	"*.pdf",
 	0|wxFD_SAVE|wxFD_OVERWRITE_PROMPT );
     my $ret = $fd->ShowModal;
-    my $fn = $fd->GetPath;
-    $fd->Destroy;
-
     if ( $ret == wxID_OK ) {
-	if ( fs_copy( $preview_file, $fn ) ) {
-	    $unsaved_preview = 0;
-	}
-	else {
-	    my $md = Wx::MessageDialog->new
-	      ( $self,
-		"Cannot save to $fn\n$!",
-		"Error saving file",
-		0 | wxOK | wxICON_ERROR);
-	    $md->ShowModal;
-	    $md->Destroy;
-	}
+	use File::Copy;
+	my $fn = $fd->GetPath;
+	copy( $preview_pdf, $fn );
+	$unsaved_preview = 0;
     }
+    $fd->Destroy;
     return $ret;
 }
 
 method have_preview {
-    fs_test( s => $preview_file );
+    -s $preview_pdf;
 }
 
 method discard {
     $unsaved_preview = 0;
-    unlink($preview_file);
+    unlink($preview_pdf);
 }
 
 1;
